package at.gv.egiz.moazs.backend;

import at.gv.egiz.moazs.client.ClientFactory;
import at.gv.egiz.moazs.repository.DeliveryRepository;
import at.gv.egiz.moazs.scheme.Msg2MzsConverter;
import at.gv.egiz.moazs.scheme.MsgResponse;
import at.gv.zustellung.app2mzs.xsd.ClientType;
import at.gv.zustellung.app2mzs.xsd.Mzs2AppPortType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.CompletableFuture;

@Component
public class ForwardResponseToServiceSink {

    private static final Logger log = LoggerFactory.getLogger(ForwardResponseToServiceSink.class);

    private final ClientFactory factory;
    private final DeliveryRepository repository;
    private final Msg2MzsConverter converter;

    @Autowired
    public ForwardResponseToServiceSink(ClientFactory factory, DeliveryRepository repository, Msg2MzsConverter converter) {
        this.factory = factory;
        this.repository = repository;
        this.converter = converter;
    }

    public CompletableFuture<Void> send(MsgResponse msgResponse, ClientType params) {
        log.info("Forward msg:{} with AppDeliveryID={} to service at {}.",
                msgResponse.getRootElementLocalPart(), msgResponse.getAppDeliveryID(), params.getURL());
        var binaryResponse = repository.retrieveBinaryResponse(msgResponse.getResponseID());
        Mzs2AppPortType client = factory.createSoap11(params, Mzs2AppPortType.class);
        return msgResponse.sendToAppClient(converter, binaryResponse, client);
    }
}
